/*
 * $Id: soundmanager.c,v 1.4 2003/02/09 07:34:16 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * BGM/SE manager.
 *
 * @version $Revision: 1.4 $
 */
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>

#include "mikmod.h"
#include "soundmanager.h"

int music_volume = 86;

static int useAudio = 0;

#define MUSIC_NUM 8

static char *musicFileName[MUSIC_NUM] = { "stg0.mod", "stg1.mod", "stg2.mod", "stg3.mod", "stg4.mod", "stg5.mod", "stg6.mod", "stg9.mod"};

//static char *musicFileName[MUSIC_NUM] = {"stg0.ogg", "stg1.ogg", "stg2.ogg", "stg3.ogg", "stg4.ogg", "stg5.ogg", "stg00.ogg"};


static MODULE *music=NULL;

#define CHUNK_NUM 7

static char *chunkFileName[CHUNK_NUM] = {
  "shot.wav", "hit.wav", "foedst.wav", "bossdst.wav", "shipdst.wav", "bonus.wav", "extend.wav",
};
static SAMPLE *chunk[CHUNK_NUM];
static int chunkFlag[CHUNK_NUM];


void closeSound() {
  int i;
  if ( !useAudio ) return;

  MikMod_DisableOutput();
  Player_Stop();
  MikMod_Update();

  if (music) {
	  Player_Free(music);
	  music = NULL;
  }

  for ( i=0 ; i<CHUNK_NUM ; i++ ) {
    if ( chunk[i] ) {
      Sample_Free(chunk[i]);
    }
  }

  MikMod_Update();
  MikMod_Exit();
}



void updateSound (void) {
  if ( !useAudio ) return;

  // note: doing this in separate thread gives no improvement
  MikMod_Update();
}

// Initialize the sound.

static void loadSounds() {
  int i;
  char name[64];

  for ( i=0 ; i<CHUNK_NUM ; i++ ) {
    strcpy(name, "data/sounds/");
    strcat(name, chunkFileName[i]);
    if ( NULL == (chunk[i] = Sample_Load(name)) ) {
      fprintf(stderr, "Couldn't load: %s\n", name);
      useAudio = 0;
      return;
    }
    chunkFlag[i] = 0;
  }

  // reserve voices for sound effects
  MikMod_SetNumVoices(-1, 4);
}

void initSound() {
  closeSound();

#ifdef WIN32	// Should be Windows
  if (drv_ds.Name)	// Valid DS driver
  {
    if (drv_ds.CommandLine)	// Valid Commandline
    {
      drv_ds.CommandLine("buffer=14");
    }
    MikMod_RegisterDriver(&drv_ds);
  }
  MikMod_RegisterDriver(&drv_win);
  MikMod_RegisterDriver(&drv_nos);
#else    // Must be Linux
  if (drv_oss.Name)  // Valid OSS driver
  {
    if (drv_oss.CommandLine)  // Valid Commandline
    {
      drv_oss.CommandLine("buffer=14,count=2");
    }
    MikMod_RegisterDriver(&drv_oss);
  }
  if (drv_alsa.Name)  // Valid ALSA driver
  {
    if (drv_alsa.CommandLine)  // Valid Commandline
    {
      drv_alsa.CommandLine("buffer=14");
    }
    MikMod_RegisterDriver(&drv_alsa);
  }
  MikMod_RegisterDriver(&drv_nos);
#endif

  // register standard tracker
  MikMod_RegisterAllLoaders();

  // Note, the md_mode flags are already set by default
  md_mode |= DMODE_SOFT_SNDFX | DMODE_SOFT_MUSIC;

  if (MikMod_Init(""))  // Command paramenters are ignored as all drivers are registered
  {
    printf("MikMod_Init failed\n");
    return;
  }


  loadSounds();

  // get ready to play
  MikMod_EnableOutput();
  md_volume = 64/2;
  //md_musicvolume = md_sndfxvolume = 64;

  useAudio = 1;
}

// Play/Stop the music/chunk.

void playMusic(int idx) {
  char name[64];
  if ( !useAudio ) return;

  Player_Stop();

  if (music) {
	  Player_Free(music);
  }
  sprintf((char*)name, "data/sounds/%s", musicFileName[idx]);

  music = Player_Load (name, 64, 0);
  if (music) {
	  printf("loaded music: \"%s\"\n", music->songname);
	  printf("%s\n", music->modtype);
	  music->fadeout = 1;
	  Player_Start (music);
	  Player_SetVolume (music_volume);
  }
}

void fadeMusic() {
  if ( !useAudio ) return;
  //Mix_FadeOutMusic(1280);
  Player_SetVolume (0);
}

void stopMusic() {
  if ( !useAudio ) return;
  Player_Stop();
}

void pauseMusic() {
  if ( !useAudio ) return;
  if (!Player_Paused())
    Player_TogglePause();
}

void unpauseMusic() {
  if ( !useAudio ) return;
  if (Player_Paused())
    Player_TogglePause();
}

void playChunk(int idx) {
  if (!useAudio) return;
  // is this safe without locking?
  int cid = Sample_Play (chunk[idx], 0, 0);
  Voice_SetPanning(cid, PAN_CENTER);
  Voice_SetVolume(cid, 256);
}

#ifdef GP2X
void increaseSoundVolume(void)
{
	if (md_volume > (128 - 1)) md_volume =  128;
	else md_volume += 1;
}

void decreaseSoundVolume(void)
{
	if (md_volume < 1) md_volume = 0;
	else md_volume -= 1;
}
#endif
